#include "usermanager.h"
#include <QTextStream>
QTextStream cout(stdout, QIODevice::WriteOnly);
QTextStream cin(stdin, QIODevice::ReadOnly);
enum  Choices {LOAD = 1, ADD, CHANGE, CHECK, SAVE, LIST, QUIT}; /* Enums are
discussed in Chap. 19. */

Choices menu() {
    QString str;
    int n;
    do {
        cout << LOAD      << "\tLoad User/PW list from file\n"
                << ADD    << "\tAdd users to the list\n"
                << CHANGE << "\tChange a user's password\n"
                << CHECK  << "\tCheck a user's password\n"
                << SAVE   << "\tSave User/PW list to file\n"
                << LIST   << "\tShow all userids and passwords\n"
                << QUIT   << "\tQuit the program\n"
                << "Your choice: " << flush;
        str = cin.readLine();
        n = str.toInt();
    } while ( n < LOAD || n > QUIT);
    return static_cast<Choices>(n); /* Casts are discussed in Chap. 19 */
}

QString promptUserid() {
    QString userid;
    int len;
    const int MINLEN(6), UMAXLEN(8);
    while (1) {
        cout << "User ID (" << MINLEN << " to " 
                << UMAXLEN << " chars - no spaces): " 
                << flush ;
         userid = cin.readLine();
         len = userid.length();
         if (len < MINLEN || len > UMAXLEN ||
             userid.contains(' ')) {
              cout << "Invalid userid!" << endl;
              continue;
         }
         return userid;
    } 
}

QString promptPW(bool repeat = true) {
    QString  pw1, pw2;
    int len;
    const int MINLEN(6), PMAXLEN(12);
    while (1) {
         cout << "Password (" << MINLEN << " to " 
                  << PMAXLEN << " chars): " << flush;
         pw1 = cin.readLine();
         len = pw1.length();
         if (len < MINLEN || len > PMAXLEN) {
             cout << "Invalid password!" << endl;
             continue;
         }
         if (repeat) {
             cout << "Retype password: " << flush;
             pw2 = cin.readLine();
             if (pw1 != pw2) {
                 cout << "Passwords do not match!" << endl;
                 continue;
             }
         }
         return pw1;
    }
}

void addUsers(UserManager& um) {
    QString userid, pw, ans;
    do {
        userid = promptUserid();
        pw = promptPW();
        cout << "Userid " << userid ;
        if (not um.addUser(userid, pw)) 
            cout << " already exists. \n" ;
        else
            cout << " added to list. \n" ;
        cout << "Add another user (y/n)? " << flush;
        ans = cin.readLine().toUpper();
    } while (ans[0] == 'Y');
}

void changePassword(UserManager& um) {
    QString userid(promptUserid());
    cout << "Current password ..." << endl;
    QString oldpw(promptPW(false));
    cout << "New password ..." << endl;
    QString newpw(promptPW());
    um.changePW(userid, oldpw, newpw);
}

void checkUser(UserManager& uw) {
    QString userid(promptUserid());
    QString pw(promptPW(false));
    if (uw.checkUser(userid, pw))
        cout << "User " << userid << " can login." <<endl;
    else
        cout << "Invalid userid or password." << endl;
}


int main() {
    QString seqstr("pspsps");
    ushort key(13579);
    UserManager um(key, seqstr);
    while (1) {
        switch (menu()) {
           case LOAD:  
                cout << "Reading from file ...\n"
                        << um.loadList() << " loaded to list"
                         << endl;
                break;
            case ADD:
                cout << "Adding users to the list ..." << endl;
                addUsers(um);
                break;
            case SAVE:
                cout << "Saving changes ...\n"
                       << um.saveList() << " users in file" << endl;
                break;
            case CHANGE:
                cout << "Changing password ..." << endl;
                changePassword(um);
                break;
            case CHECK:
                cout << "Checking a userid/pw combo ..." << endl;
                checkUser(um);
                break;
            case LIST:
                cout << "Listing users and passwords ...\n";
                um.listUsers();
                break;
            case QUIT:
                cout << "Exiting the program ..." << endl;
                return 0;
           default:
                cout << "Invalid choice! " << endl;
        }
    }
}

